"""
StoredFile has moved to qp.lib.stored_file.
This module should be removed after all existing StoredFiles are repickled
using the correct import.
"""
from qp.lib.stored_file import StoredFile

StoredFile # quiet checker
print("Please repickle all StoredFile instances and references.")

