"""
open/dulcinea/lib/session.py
"""
from dulcinea.attachable import Attachable
from dulcinea.user import DulcineaUser
from qp.lib.spec import Specified
from qp.pub.session import Session


class DulcineaSession (Session, Specified, Attachable):

    effective_user_is = DulcineaUser
    owner_is = DulcineaUser

    def __init__(self):
        Session.__init__(self)
        Attachable.__init__(self)

    def clear_app_state(self):
        """Override to clear any session data
        """
    def set_items_per_page(self, value):
        """Override if you want this to really save something.
        """

    def get_items_per_page(self):
        return 10
